/**
    (C) Copyright 2012 Dominik Wagenfuehr

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "Game.hh"
#include "Bot.hh"
#include "StrategyFactory.hh"

#include <iostream>
#include <sstream>

// Startet das Programm.
bool Game::start( const std::string& strategy )
{
    bool retValue = false;

    // Strategie von der Factory erstellen lassen.
    IStrategy* strategyP = StrategyFactory::getInstance().create( strategy );

    // Wurde eine Strategie erstellt?
    if ( strategyP )
    {
        retValue = true;

        // Bot erstellen.
        Bot bot;

        // Strategy setzen.
        bot.setStrategy( strategyP );

        std::string input;
        int value = 0;
        do {
            // Eingabe vom Benutzer lesen
            input = readInput();

            // Eingabe in Zahl wandeln
            value = parseInput( input );

            if ( 0 <= value && value <= 1000 )
            {
                if ( bot.acceptOffer( value) )
                {
                    std::cout << "Bot nimmt Angebot an." << std::endl;
                }
                else
                {
                    std::cout << "Bot lehnt Angebot ab." << std::endl;
                }
            }
            // wiederhole, bis Wert ungueltig ist
        } while ( 0 <= value && value <= 1000 );

        // Das Spiel ist zu Ende, wir geben die Punkte aus.
        std::cout << "Gesammelte Punkte: " << bot.getPoints() << std::endl;
    }
    else
    {
        std::cout << "Strategie unbekannt oder anderer Fehler." << std::endl;
    }

    return retValue;
}

// Liest eine Eingabe vom Benutzer.
std::string Game::readInput() const
{
    std::string input;
    std::cout << "Zahl eingeben: ";
    getline(std::cin, input);
    return input;
}

// Interpretiert die Benutzereingabe.
int Game::parseInput( const std::string& input ) const
{
    // String in Integer konvertieren
    int value = 0;
    std::istringstream in(input);
    in >> value;
    return value;
}

